from collections import OrderedDict

def merge_dictionaries(*args):
    if len(args) < 1:
        return {}
    z = args[0].copy()
    for a in args[1:]:
        z.update(a)
    return z

def merge_ordered_dictionaries(*args):
    if len(args) < 1:
        return {}
    z = OrderedDict(args[0].copy())
    for a in args[1:]:
        z.update(a)
    return z


class Map(dict):
    def __init__(self, *args, **kwargs):
        super(Map, self).__init__(*args, **kwargs)
        for arg in args:
            if isinstance(arg, dict):
                for k, v in arg.items():
                    self[k] = v

        if kwargs:
            for k, v in kwargs.items():
                self[k] = v

    def __getattr__(self, attr):
        return self.get(attr)

    def __setattr__(self, key, value):
        self.__setitem__(key, value)

    def __setitem__(self, key, value):
        super(Map, self).__setitem__(key, value)
        self.__dict__.update({key: value})

    def __delattr__(self, item):
        self.__delitem__(item)

    def __delitem__(self, key):
        super(Map, self).__delitem__(key)
        del self.__dict__[key]